@@ -4,13 +4,10 @@ use full_moon::ast::{
     Call, Expression, FunctionArgs, FunctionBody, FunctionCall, FunctionDeclaration, FunctionName,
     LocalFunction, MethodCall, Parameter, Value,
 };
-use full_moon::node::Node;
 use full_moon::tokenizer::{Symbol, Token, TokenKind, TokenReference, TokenType};
-use std::borrow::Cow;
 use std::boxed::Box;
 
 use crate::formatters::{
-    get_line_ending_character,
     trivia_formatter::{self, FormatTriviaType},
     trivia_util, CodeFormatter,
 };
@@ -22,7 +19,7 @@ impl CodeFormatter {
         &mut self,
         function_token: &TokenReference<'ast>,
         function_body: &FunctionBody<'ast>,
-    ) -> (Cow<'ast, TokenReference<'ast>>, FunctionBody<'ast>) {
+    ) -> (TokenReference<'ast>, FunctionBody<'ast>) {
         let function_token_range = CodeFormatter::get_token_range(function_token.token());
         let additional_indent_level = self.get_range_indent_increase(function_token_range); //code_formatter.get_token_indent_increase(function_token.token());
 
@@ -66,11 +63,11 @@ impl CodeFormatter {
             function_body = function_body.with_parameters_parentheses(parameters_parentheses);
         };
 
-        let end_token = Cow::Owned(trivia_formatter::token_reference_add_trivia(
+        let end_token = trivia_formatter::token_reference_add_trivia(
             function_body.end_token().to_owned(),
             FormatTriviaType::Append(vec![self.create_indent_trivia(additional_indent_level)]),
             FormatTriviaType::NoChange,
-        ));
+        );
 
         (function_token, function_body.with_end_token(end_token))
     }
@@ -82,6 +79,7 @@ impl CodeFormatter {
                 Call::AnonymousCall(self.format_function_args(function_args))
             }
             Call::MethodCall(method_call) => Call::MethodCall(self.format_method_call(method_call)),
+            other => panic!("unknown node {:?}", other),
         }
     }
 
@@ -101,17 +99,17 @@ impl CodeFormatter {
                     Token::end_position(&start_parens).bytes(),
                     Token::start_position(&end_parens).bytes(),
                 );
-                let mut is_multiline =
-                    // We subtract 20 as we don't have full information about what preceded these function arguments (e.g. the function name).
-                    // This is used as a general estimate. TODO: see if we can improve this calculation
-                    self.get_indent_width() + (function_call_range.1 - function_call_range.0) > self.config.column_width - 20;
-                let current_arguments = arguments.pairs();
+                let current_indent_width = self.get_indent_width()
+                    + (self
+                        .get_range_indent_increase(CodeFormatter::get_token_range(start_parens))
+                        .unwrap_or(0)
+                        * self.config.indent_width);
 
                 // Format all the arguments, so that we can prepare them and check to see whether they need expanding
                 // We will ignore punctuation for now
-                let mut formatted_arguments = Vec::new();
+                let mut first_iter_formatted_arguments = Vec::new();
                 for argument in arguments.iter() {
-                    formatted_arguments.push(self.format_expression(argument))
+                    first_iter_formatted_arguments.push(self.format_expression(argument))
                 }
 
                 // Apply some heuristics to determine whether we should expand the function call
@@ -156,70 +154,118 @@ impl CodeFormatter {
                         contains_comments
                     };
 
-                if force_mutliline {
-                    is_multiline = true;
-                }
+                let mut is_multiline = force_mutliline;
 
-                if is_multiline && !force_mutliline {
+                if !force_mutliline {
                     // If we only have one argument then we will not make it multi line (expanding it would have little value)
                     // Unless, the argument is a hangable expression
-                    if formatted_arguments.len() == 1
-                        && !trivia_util::can_hang_expression(formatted_arguments.first().unwrap())
+                    if first_iter_formatted_arguments.len() == 1
+                        && !trivia_util::can_hang_expression(
+                            first_iter_formatted_arguments.first().unwrap(),
+                        )
                     {
                         is_multiline = false;
                     } else {
                         // Find how far we are currently indented, we can use this to determine when to expand
+                        // We will expand on two occasions:
+                        // 1) If a group of arguments fall on a single line, and they surpass the column width setting
+                        // 2) If we have a mixture of multiline (tables/anonymous functions) and other values. For
+                        //    example, call({ ... }, foo, { ... }), should be expanded, but
+                        //    call(foo, { ... }) or call(foo, { ... }, foo) can stay on one line, provided the
+                        //    single line arguments dont surpass the column width setting
+
                         // TODO: We need to add more to this - there may be quite a lot before this function call
-                        // TODO: include additional_indent_level
-                        let mut width_passed = self.get_indent_width();
-                        let mut keep_single_line = false;
-                        // Check to see if we have a table constructor, or anonymous function
-                        for argument in formatted_arguments.iter() {
-                            // TODO: Do we need to worry about parentheses or UnOp?
-                            if let Expression::Value { value, .. } = argument {
-                                match &**value {
-                                    Value::Function(_) => {
-                                        // An anonymous function should always be expanded
-                                        // This is safe to prevent expansion
-                                        keep_single_line = true;
-                                    }
-                                    Value::TableConstructor(table) => {
-                                        // Check to see whether it has been expanded
-                                        let start_brace = table.braces().tokens().0;
-                                        let is_expanded = trivia_util::trivia_contains_newline(
-                                            start_brace.trailing_trivia(),
-                                        );
-                                        if is_expanded {
-                                            keep_single_line = true
-                                        } else {
-                                            // We have a collapsed table constructor - add the width, and if it fails,
-                                            // we need to expand
+                        let mut width_passed = current_indent_width;
+
+                        // Use state values to determine the type of arguments we have seen so far
+                        let mut seen_multiline_arg = false; // Whether we have seen a multiline table/function already
+                        let mut seen_other_arg_after_multiline = false; // Whether we have seen a non multiline table/function after a multiline one. In this case, we should expand
+
+                        for argument in first_iter_formatted_arguments.iter() {
+                            match argument {
+                                Expression::Value { value, .. } => {
+                                    match &**value {
+                                        // Check to see if we have a table constructor, or anonymous function
+                                        Value::Function(_) => {
+                                            // If we have a mixture of multiline args, and other arguments
+                                            // Then the function args should be expanded
+                                            if seen_multiline_arg && seen_other_arg_after_multiline
+                                            {
+                                                is_multiline = true;
+                                                break;
+                                            }
+
+                                            seen_multiline_arg = true;
+
+                                            // Reset the width count back
+                                            width_passed = current_indent_width;
+                                        }
+                                        Value::TableConstructor(table) => {
+                                            // Check to see whether it has been expanded
+                                            let start_brace = table.braces().tokens().0;
+                                            let is_expanded = trivia_util::trivia_contains_newline(
+                                                start_brace.trailing_trivia(),
+                                            );
+                                            if is_expanded {
+                                                // If we have a mixture of multiline args, and other arguments
+                                                // Then the function args should be expanded
+                                                if seen_multiline_arg
+                                                    && seen_other_arg_after_multiline
+                                                {
+                                                    is_multiline = true;
+                                                    break;
+                                                }
+
+                                                seen_multiline_arg = true;
+                                                // Reset the width count back
+                                                width_passed = current_indent_width;
+                                            } else {
+                                                // We have a collapsed table constructor - add the width, and if it fails,
+                                                // we need to expand
+                                                width_passed += argument.to_string().len();
+                                                if width_passed > self.config.column_width - 20 {
+                                                    is_multiline = true;
+                                                    break;
+                                                }
+                                            }
+                                        }
+                                        _ => {
+                                            // If we previously had a table/anonymous function, and we have something else
+                                            // in the mix, update the state to respond to this
+                                            if seen_multiline_arg {
+                                                seen_other_arg_after_multiline = true;
+                                            }
                                             width_passed += argument.to_string().len();
                                             if width_passed > self.config.column_width - 20 {
+                                                // We have passed 80 characters without a table or anonymous function
+                                                // There is nothing else stopping us from expanding - so we will
+                                                is_multiline = true;
                                                 break;
                                             }
                                         }
                                     }
-                                    _ => {
-                                        // If we previously had a table/anonymous function, and we have something else
-                                        // in the mix, we should not expand
-                                        if keep_single_line {
-                                            keep_single_line = false;
-                                            break;
-                                        }
-                                        width_passed += argument.to_string().len();
-                                        if width_passed > self.config.column_width - 20 {
-                                            // We have passed 80 characters without a table or anonymous function
-                                            // There is nothing else stopping us from expanding - so we will
-                                            break;
-                                        }
+                                }
+                                // TODO: Parentheses/UnOp, do we need to do more checking?
+                                // We will continue counting on the width_passed
+                                _ => {
+                                    // If we previously had a table/anonymous function, and we have something else
+                                    // in the mix, update the state to respond to this
+                                    if seen_multiline_arg {
+                                        seen_other_arg_after_multiline = true;
+                                    }
+
+                                    width_passed += argument.to_string().len();
+                                    if width_passed > self.config.column_width - 20 {
+                                        // We have passed 80 characters without a table or anonymous function
+                                        // There is nothing else stopping us from expanding - so we will
+                                        is_multiline = true;
+                                        break;
                                     }
                                 }
                             }
-                        }
 
-                        if keep_single_line {
-                            is_multiline = false;
+                            // Add width which would be taken up by comment and space
+                            width_passed += 2;
                         }
                     }
                 }
@@ -239,7 +285,7 @@ impl CodeFormatter {
                     // Add new_line trivia to start_parens
                     let start_parens_token = crate::fmt_symbol!(self, start_parens, "(");
                     let start_parens_token = trivia_formatter::token_reference_add_trivia(
-                        start_parens_token.into_owned(),
+                        start_parens_token,
                         FormatTriviaType::NoChange,
                         FormatTriviaType::Append(vec![self.create_newline_trivia()]),
                     );
@@ -253,17 +299,17 @@ impl CodeFormatter {
                     );
 
                     let parentheses = ContainedSpan::new(
-                        Cow::Owned(start_parens_token),
+                        start_parens_token,
                         self.format_symbol(end_parens, &end_parens_token),
                     );
 
-                    let mut arguments = Punctuated::new();
+                    let mut formatted_arguments = Punctuated::new();
                     // Iterate through the original formatted_arguments, so we can see how the formatted version would look like
-                    let mut formatted_versions = formatted_arguments.iter();
+                    let mut formatted_versions = first_iter_formatted_arguments.iter();
 
                     self.add_indent_range(function_call_range);
 
-                    for argument in current_arguments {
+                    for argument in arguments.pairs() {
                         // See what the formatted version of the argument would look like
                         let formatted_version = formatted_versions
                             .next()
@@ -288,13 +334,6 @@ impl CodeFormatter {
                                         .len()
                                     > self.config.column_width;
 
-                        if require_multiline_expression {
-                            let expr_range = argument
-                                .range()
-                                .expect("no range for function call argument");
-                            self.add_indent_range((expr_range.0.bytes(), expr_range.1.bytes() + 1));
-                        }
-
                         // Unfortunately, we need to format again, taking into account in indent increase
                         // TODO: Can we fix this? We don't want to have to format twice
                         let mut formatted_argument = self.format_expression(argument.value());
@@ -321,30 +360,26 @@ impl CodeFormatter {
                                 // Continue adding a comma and a new line for multiline function args
                                 let symbol = crate::fmt_symbol!(self, punctuation, ",");
                                 let symbol = trivia_formatter::token_reference_add_trivia(
-                                    symbol.into_owned(),
+                                    symbol,
                                     FormatTriviaType::NoChange,
                                     FormatTriviaType::Append(vec![self.create_newline_trivia()]),
                                 );
 
-                                Some(Cow::Owned(symbol))
+                                Some(symbol)
                             }
-                            None => Some(Cow::Owned(TokenReference::new(
+                            None => Some(TokenReference::new(
                                 vec![],
-                                Token::new(TokenType::Whitespace {
-                                    characters: Cow::Owned(get_line_ending_character(
-                                        &self.config.line_endings,
-                                    )),
-                                }),
+                                self.create_newline_trivia(),
                                 vec![],
-                            ))),
+                            )),
                         };
 
-                        arguments.push(Pair::new(formatted_argument, punctuation))
+                        formatted_arguments.push(Pair::new(formatted_argument, punctuation))
                     }
 
                     FunctionArgs::Parentheses {
                         parentheses,
-                        arguments,
+                        arguments: formatted_arguments,
                     }
                 } else {
                     let parentheses = self.format_contained_span(&parentheses);
@@ -373,16 +408,16 @@ impl CodeFormatter {
                     // Recreate parentheses with the comments removed from the opening parens
                     // and all the comments placed at the end of the closing parens
                     let parentheses = ContainedSpan::new(
-                        Cow::Owned(trivia_formatter::token_reference_add_trivia(
+                        trivia_formatter::token_reference_add_trivia(
                             start_parens.to_owned(),
                             FormatTriviaType::NoChange,
                             FormatTriviaType::Replace(vec![]),
-                        )),
-                        Cow::Owned(trivia_formatter::token_reference_add_trivia(
+                        ),
+                        trivia_formatter::token_reference_add_trivia(
                             end_parens.to_owned(),
                             FormatTriviaType::NoChange,
                             FormatTriviaType::Append(parens_comments),
-                        )),
+                        ),
                     );
 
                     FunctionArgs::Parentheses {
@@ -396,9 +431,8 @@ impl CodeFormatter {
                 let mut arguments = Punctuated::new();
                 let new_expression = self.format_expression(&Expression::Value {
                     value: Box::new(Value::String(token_reference.to_owned())),
-                    binop: None,
                     #[cfg(feature = "luau")]
-                    as_assertion: None,
+                    type_assertion: None,
                 });
 
                 // Remove any trailing comments from the expression, and move them into a buffer
@@ -408,8 +442,8 @@ impl CodeFormatter {
                 // Create parentheses, and add the trailing comments to the end of the parentheses
                 let parentheses = trivia_formatter::contained_span_add_trivia(
                     ContainedSpan::new(
-                        Cow::Owned(TokenReference::symbol("(").unwrap()),
-                        Cow::Owned(TokenReference::symbol(")").unwrap()),
+                        TokenReference::symbol("(").unwrap(),
+                        TokenReference::symbol(")").unwrap(),
                     ),
                     FormatTriviaType::NoChange,
                     FormatTriviaType::Append(comments_buffer),
@@ -427,9 +461,8 @@ impl CodeFormatter {
                 let mut arguments = Punctuated::new();
                 let new_expression = self.format_expression(&Expression::Value {
                     value: Box::new(Value::TableConstructor(table_constructor.to_owned())),
-                    binop: None,
                     #[cfg(feature = "luau")]
-                    as_assertion: None,
+                    type_assertion: None,
                 });
 
                 // Remove any trailing comments from the expression, and move them into a buffer
@@ -439,8 +472,8 @@ impl CodeFormatter {
                 // Create parentheses, and add the trailing comments to the end of the parentheses
                 let parentheses = trivia_formatter::contained_span_add_trivia(
                     ContainedSpan::new(
-                        Cow::Owned(TokenReference::symbol("(").unwrap()),
-                        Cow::Owned(TokenReference::symbol(")").unwrap()),
+                        TokenReference::symbol("(").unwrap(),
+                        TokenReference::symbol(")").unwrap(),
                     ),
                     FormatTriviaType::NoChange,
                     FormatTriviaType::Append(comments_buffer),
@@ -453,6 +486,7 @@ impl CodeFormatter {
                     arguments,
                 }
             }
+            other => panic!("unknown node {:?}", other),
         }
     }
 
@@ -489,7 +523,7 @@ impl CodeFormatter {
                 // Add new_line trivia to start_parens
                 let start_parens_token = crate::fmt_symbol!(self, start_parens, "(");
                 let start_parens_token = trivia_formatter::token_reference_add_trivia(
-                    start_parens_token.into_owned(),
+                    start_parens_token,
                     FormatTriviaType::NoChange,
                     FormatTriviaType::Append(vec![self.create_newline_trivia()]),
                 );
@@ -503,7 +537,7 @@ impl CodeFormatter {
                 );
 
                 ContainedSpan::new(
-                    Cow::Owned(start_parens_token),
+                    start_parens_token,
                     self.format_symbol(end_parens, &end_parens_token),
                 )
             }
@@ -554,12 +588,11 @@ impl CodeFormatter {
         }
 
         let end_token = if add_trivia {
-            Cow::Owned(trivia_formatter::token_reference_add_trivia(
-                self.format_end_token(function_body.end_token())
-                    .into_owned(),
+            trivia_formatter::token_reference_add_trivia(
+                self.format_end_token(function_body.end_token()),
                 FormatTriviaType::Append(leading_trivia),
                 FormatTriviaType::Append(trailing_trivia),
-            ))
+            )
         } else {
             self.format_end_token(function_body.end_token())
         };
@@ -585,7 +618,7 @@ impl CodeFormatter {
     ) -> FunctionCall<'ast> {
         let formatted_prefix = self.format_prefix(function_call.prefix());
         let formatted_suffixes = function_call
-            .iter_suffixes()
+            .suffixes()
             .map(|x| self.format_suffix(x))
             .collect();
 
@@ -614,16 +647,13 @@ impl CodeFormatter {
             }
         }
 
-        let mut formatted_method: Option<(
-            Cow<'ast, TokenReference<'ast>>,
-            Cow<'ast, TokenReference<'ast>>,
-        )> = None;
+        let mut formatted_method: Option<(TokenReference<'ast>, TokenReference<'ast>)> = None;
 
         if let Some(method_colon) = function_name.method_colon() {
             if let Some(token_reference) = function_name.method_name() {
                 formatted_method = Some((
                     crate::fmt_symbol!(self, method_colon, ":"),
-                    Cow::Owned(self.format_plain_token_reference(token_reference)),
+                    self.format_token_reference(token_reference),
                 ));
             }
         };
@@ -642,12 +672,11 @@ impl CodeFormatter {
         );
         let leading_trivia = vec![self.create_indent_trivia(additional_indent_level)];
 
-        let function_token = Cow::Owned(trivia_formatter::token_reference_add_trivia(
-            crate::fmt_symbol!(self, function_declaration.function_token(), "function ")
-                .into_owned(),
+        let function_token = trivia_formatter::token_reference_add_trivia(
+            crate::fmt_symbol!(self, function_declaration.function_token(), "function "),
             FormatTriviaType::Append(leading_trivia),
             FormatTriviaType::NoChange,
-        ));
+        );
         let formatted_function_name = self.format_function_name(function_declaration.name());
         let formatted_function_body = self.format_function_body(function_declaration.body(), true);
 
@@ -667,15 +696,15 @@ impl CodeFormatter {
         );
         let leading_trivia = vec![self.create_indent_trivia(additional_indent_level)];
 
-        let local_token = Cow::Owned(trivia_formatter::token_reference_add_trivia(
-            crate::fmt_symbol!(self, local_function.local_token(), "local ").into_owned(),
+        let local_token = trivia_formatter::token_reference_add_trivia(
+            crate::fmt_symbol!(self, local_function.local_token(), "local "),
             FormatTriviaType::Append(leading_trivia),
             FormatTriviaType::NoChange,
-        ));
+        );
 
         let function_token = crate::fmt_symbol!(self, local_function.function_token(), "function ");
-        let formatted_name = Cow::Owned(self.format_plain_token_reference(local_function.name()));
-        let formatted_function_body = self.format_function_body(local_function.func_body(), true);
+        let formatted_name = self.format_token_reference(local_function.name());
+        let formatted_function_body = self.format_function_body(local_function.body(), true);
 
         LocalFunction::new(formatted_name)
             .with_local_token(local_token)
@@ -685,12 +714,12 @@ impl CodeFormatter {
 
     /// Formats a MethodCall node
     pub fn format_method_call<'ast>(&mut self, method_call: &MethodCall<'ast>) -> MethodCall<'ast> {
-        let formatted_colon_token = self.format_plain_token_reference(method_call.colon_token());
-        let formatted_name = self.format_plain_token_reference(method_call.name());
+        let formatted_colon_token = self.format_token_reference(method_call.colon_token());
+        let formatted_name = self.format_token_reference(method_call.name());
         let formatted_function_args = self.format_function_args(method_call.args());
 
-        MethodCall::new(Cow::Owned(formatted_name), formatted_function_args)
-            .with_colon_token(Cow::Owned(formatted_colon_token))
+        MethodCall::new(formatted_name, formatted_function_args)
+            .with_colon_token(formatted_colon_token)
     }
 
     /// Formats a single Parameter node
@@ -700,16 +729,17 @@ impl CodeFormatter {
             Parameter::Name(token_reference) => {
                 Parameter::Name(self.format_token_reference(token_reference))
             }
+            other => panic!("unknown node {:?}", other),
         }
     }
 
     // Checks whether the input Parameter contains comments
     fn parameter_contains_comments(parameter: &Parameter<'_>) -> bool {
         match parameter {
-            Parameter::Ellipse(token) | Parameter::Name(token) => match token {
-                Cow::Owned(t) => trivia_util::token_contains_comments(&t),
-                Cow::Borrowed(t) => trivia_util::token_contains_comments(t),
-            },
+            Parameter::Ellipse(token) | Parameter::Name(token) => {
+                trivia_util::token_contains_comments(token)
+            }
+            other => panic!("unknown node {:?}", other),
         }
     }
     /// Utilises the FunctionBody iterator to format a list of Parameter nodes
@@ -742,12 +772,12 @@ impl CodeFormatter {
 
             let formatted_punctuation = match pair.punctuation() {
                 Some(punctuation) => Some(match force_multiline {
-                    true => Cow::Owned(trivia_formatter::token_reference_add_trivia(
+                    true => trivia_formatter::token_reference_add_trivia(
                         // Create a comma with no trailing space, and instead we will add a newline character
-                        crate::fmt_symbol!(self, punctuation, ",").into_owned(),
+                        crate::fmt_symbol!(self, punctuation, ","),
                         FormatTriviaType::NoChange,
                         FormatTriviaType::Append(vec![self.create_newline_trivia()]),
-                    )),
+                    ),
                     // Create a comma, with a trailing space at the end
                     false => crate::fmt_symbol!(self, punctuation, ", "),
                 }),
