@@ -1,13 +1,11 @@
-use crate::{Config, IndentType, LineEndings};
+use crate::{Config, IndentType, LineEndings, QuoteStyle};
 use full_moon::ast::{
     punctuated::{Pair, Punctuated},
     span::ContainedSpan,
     Block,
 };
 use full_moon::node::Node;
-use full_moon::tokenizer::{
-    StringLiteralQuoteType, Symbol, Token, TokenKind, TokenReference, TokenType,
-};
+use full_moon::tokenizer::{StringLiteralQuoteType, Token, TokenKind, TokenReference, TokenType};
 use full_moon::visitors::VisitorMut;
 use std::borrow::Cow;
 use std::collections::HashSet;
@@ -17,6 +15,8 @@ pub mod block_formatter;
 #[macro_use]
 pub mod expression_formatter;
 pub mod functions_formatter;
+#[cfg(feature = "lua52")]
+pub mod lua52_formatter;
 #[cfg(feature = "luau")]
 pub mod luau_formatter;
 pub mod stmt_formatter;
@@ -200,12 +200,40 @@ impl CodeFormatter {
         comment.trim_end().to_string()
     }
 
+    fn get_quote_to_use(&self, literal: &str) -> StringLiteralQuoteType {
+        match self.config.quote_style {
+            QuoteStyle::ForceDouble => StringLiteralQuoteType::Double,
+            QuoteStyle::ForceSingle => StringLiteralQuoteType::Single,
+            _ => {
+                let preferred = match self.config.quote_style {
+                    QuoteStyle::AutoPreferDouble => StringLiteralQuoteType::Double,
+                    QuoteStyle::AutoPreferSingle => StringLiteralQuoteType::Single,
+                    _ => unreachable!("have other quote styles we haven't looked into yet"),
+                };
+
+                // Check to see if there is a quote within it
+                if literal.contains('\'') || literal.contains('"') {
+                    let num_single_quotes = literal.matches('\'').count();
+                    let num_double_quotes = literal.matches('"').count();
+
+                    match num_single_quotes.cmp(&num_double_quotes) {
+                        std::cmp::Ordering::Equal => preferred,
+                        std::cmp::Ordering::Greater => StringLiteralQuoteType::Double,
+                        std::cmp::Ordering::Less => StringLiteralQuoteType::Single,
+                    }
+                } else {
+                    preferred
+                }
+            }
+        }
+    }
+
     /// Formats a Token Node
     /// Also returns any extra leading or trailing trivia to add for the Token node
     /// This should only ever be called from format_token_reference
     fn format_token<'ast>(
         &self,
-        token: Token<'ast>,
+        token: &Token<'ast>,
         format_type: &FormatTokenType,
         additional_indent_level: Option<usize>,
     ) -> (
@@ -246,27 +274,37 @@ impl CodeFormatter {
                     // Match all escapes within the the string
                     // Based off https://github.com/prettier/prettier/blob/181a325c1c07f1a4f3738665b7b28288dfb960bc/src/common/util.js#L439
                     lazy_static::lazy_static! {
-                        static ref RE: regex::Regex = regex::Regex::new(r#"\\([\S\s])|(["'])"#).unwrap();
-                        static ref UNNECESSARY_ESCAPES: regex::Regex = regex::Regex::new(r#"^[^\n\r"'0-7\\bfnrt-vx\u2028\u2029]$"#).unwrap();
+                        static ref RE: regex::Regex = regex::Regex::new(r#"\\?(["'])|\\([\S\s])"#).unwrap();
+                        static ref UNNECESSARY_ESCAPES: regex::Regex = regex::Regex::new(r#"^[^\n\r"'0-7\\abfnrtuvxz]$"#).unwrap();
                     }
+                    let quote_to_use = self.get_quote_to_use(literal);
                     let literal = RE
                         .replace_all(literal, |caps: &regex::Captures| {
-                            let escaped = caps.get(1);
-                            let quote = caps.get(2);
+                            let quote = caps.get(1);
+                            let escaped = caps.get(2);
 
                             match quote {
                                 Some(quote) => {
-                                    // We have a quote, so lets see if it matches what we want, and
-                                    let quote_type = match quote.as_str() {
-                                        "'" => StringLiteralQuoteType::Single,
-                                        "\"" => StringLiteralQuoteType::Double,
-                                        _ => panic!("unknown quote type"),
-                                    };
-                                    if let StringLiteralQuoteType::Single = quote_type {
-                                        String::from("'")
-                                    } else {
-                                        // Double quote, make sure to escape it
-                                        String::from("\\\"")
+                                    // We have a quote, find what type it is, and see if we need to escape it
+                                    // then return the output string
+                                    match quote.as_str() {
+                                        "'" => {
+                                            // Check whether to escape the quote
+                                            if let StringLiteralQuoteType::Single = quote_to_use {
+                                                String::from("\\'")
+                                            } else {
+                                                String::from("'")
+                                            }
+                                        }
+                                        "\"" => {
+                                            // Check whether to escape the quote
+                                            if let StringLiteralQuoteType::Double = quote_to_use {
+                                                String::from("\\\"")
+                                            } else {
+                                                String::from("\"")
+                                            }
+                                        }
+                                        other => unreachable!("unknown quote type {:?}", other),
                                     }
                                 }
                                 None => {
@@ -289,7 +327,7 @@ impl CodeFormatter {
                     TokenType::StringLiteral {
                         literal: Cow::Owned(literal),
                         multi_line: None,
-                        quote_type: StringLiteralQuoteType::Double,
+                        quote_type: quote_to_use,
                     }
                 }
             }
@@ -368,25 +406,45 @@ impl CodeFormatter {
         let mut token_trivia = Vec::new();
 
         let mut newline_count_in_succession = 0;
+        let mut trivia_iter = current_trivia.iter().peekable();
 
-        for trivia in current_trivia {
+        while let Some(trivia) = trivia_iter.next() {
+            // Handle cases where the user has left a newline gap in between e.g. two statements
+            // If we are formatting trailing trivia, this can be ignored, as all trailing newlines will have already
+            // been handled by the formatter.
+            // If we are formatting leading trivia, we will allow a single newline to be kept in succession, if we
+            // find one.
             match trivia.token_type() {
                 TokenType::Whitespace { characters } => {
-                    if characters.contains('\n') {
-                        newline_count_in_succession += 1;
-                        if newline_count_in_succession == 2 {
-                            // We have two counts of a new line, we will allow one to be kept
-                            // This allows the user to define where they want to keep lines in between statements, whilst only allowing one empty line in between them
-                            token_trivia.push(Token::new(TokenType::Whitespace {
-                                characters: Cow::Owned(get_line_ending_character(
-                                    &self.config.line_endings,
-                                )),
-                            }));
+                    if let FormatTokenType::LeadingTrivia = format_token_type {
+                        if characters.contains('\n') {
+                            newline_count_in_succession += 1;
+                            if newline_count_in_succession == 1 {
+                                // We have a case where we will allow a single newline to be kept
+                                token_trivia.push(self.create_newline_trivia());
+                            }
                         }
                     }
+
                     // Move to next trivia
                     continue;
                 }
+                TokenType::SingleLineComment { .. } | TokenType::MultiLineComment { .. } => {
+                    // If we have a comment, when `format_token` is called, it will put a newline at the end
+                    // If this happens, we want to skip the next iteration if its a newline, as that has already been covered here
+                    if let FormatTokenType::LeadingTrivia = format_token_type {
+                        if let Some(next_trivia) = trivia_iter.peek() {
+                            if let TokenType::Whitespace { characters } = next_trivia.token_type() {
+                                if characters.contains('\n') {
+                                    // Consume iterator once to skip the next iteration
+                                    trivia_iter.next();
+                                }
+                            }
+                        }
+                    }
+                    // We will reset the counter as well, because the newline above is only to terminate the comment
+                    newline_count_in_succession = 0;
+                }
                 _ => {
                     // Reset new line counter, as we only want two new lines in a row
                     newline_count_in_succession = 0;
@@ -412,7 +470,7 @@ impl CodeFormatter {
         token_trivia
     }
 
-    fn format_plain_token_reference<'a>(
+    fn format_token_reference<'a>(
         &self,
         token_reference: &TokenReference<'a>,
     ) -> TokenReference<'a> {
@@ -428,11 +486,8 @@ impl CodeFormatter {
             None,
         );
 
-        let (token, _leading_trivia, _trailing_trivia) = self.format_token(
-            token_reference.token().to_owned(),
-            &FormatTokenType::Token,
-            None,
-        );
+        let (token, _leading_trivia, _trailing_trivia) =
+            self.format_token(token_reference.token(), &FormatTokenType::Token, None);
         // TODO: is it possible for leading/trailing trivia to be present here?
         // if let Some(trivia) = leading_trivia {
         //     formatted_leading_trivia.append(trivia);
@@ -445,18 +500,11 @@ impl CodeFormatter {
         TokenReference::new(formatted_leading_trivia, token, formatted_trailing_trivia)
     }
 
-    pub fn format_token_reference<'a>(
-        &self,
-        token_reference: &TokenReference<'a>,
-    ) -> Cow<'a, TokenReference<'a>> {
-        Cow::Owned(self.format_plain_token_reference(&token_reference))
-    }
-
     pub fn format_token_reference_mut<'ast>(
         &mut self,
-        token_reference: &Cow<'ast, TokenReference<'ast>>,
-    ) -> Cow<'ast, TokenReference<'ast>> {
-        Cow::Owned(self.format_plain_token_reference(&token_reference))
+        token_reference: &TokenReference<'ast>,
+    ) -> TokenReference<'ast> {
+        self.format_token_reference(token_reference)
     }
 
     // Formats a punctuation for a Punctuated sequence
@@ -464,7 +512,7 @@ impl CodeFormatter {
     pub fn format_punctuation<'ast>(
         &self,
         punctuation: &TokenReference<'ast>,
-    ) -> (Cow<'ast, TokenReference<'ast>>, Vec<Token<'ast>>) {
+    ) -> (TokenReference<'ast>, Vec<Token<'ast>>) {
         let trailing_comments = punctuation
             .trailing_trivia()
             .filter(|token| {
@@ -479,11 +527,11 @@ impl CodeFormatter {
             .collect();
 
         (
-            Cow::Owned(TokenReference::new(
+            TokenReference::new(
                 Vec::new(),
                 punctuation.token().to_owned(),
                 vec![Token::new(TokenType::spaces(1))], // Single space whitespace
-            )),
+            ),
             trailing_comments,
         )
     }
@@ -527,8 +575,8 @@ impl CodeFormatter {
         let (start_token, end_token) = contained_span.tokens();
 
         ContainedSpan::new(
-            Cow::Owned(self.format_plain_token_reference(start_token)),
-            Cow::Owned(self.format_plain_token_reference(end_token)),
+            self.format_token_reference(start_token),
+            self.format_token_reference(end_token),
         )
     }
 
@@ -538,7 +586,7 @@ impl CodeFormatter {
         &self,
         current_symbol: &TokenReference<'ast>,
         wanted_symbol: &TokenReference<'ast>,
-    ) -> Cow<'ast, TokenReference<'ast>> {
+    ) -> TokenReference<'ast> {
         // Preserve comments in leading/trailing trivia
         let mut formatted_leading_trivia: Vec<Token<'ast>> = self.load_token_trivia(
             current_symbol.leading_trivia().collect(),
@@ -566,19 +614,19 @@ impl CodeFormatter {
         wanted_trailing_trivia.append(&mut formatted_trailing_trivia);
         formatted_leading_trivia.append(&mut wanted_leading_trivia);
 
-        Cow::Owned(TokenReference::new(
+        TokenReference::new(
             formatted_leading_trivia,
             wanted_symbol.token().to_owned(),
             wanted_trailing_trivia,
-        ))
+        )
     }
 
-    /// Formats an `end` token, which is normally present to close a block
-    /// This is required due to comments bound to an `end` token - they need to have one level higher indentation
+    /// Formats a token present at the end of an indented block, such as the `end` token or closing brace in a multiline table.
+    /// This is required due to leading comments bound to the last token - they need to have one level higher indentation
     pub fn format_end_token<'ast>(
         &self,
         current_token: &TokenReference<'ast>,
-    ) -> Cow<'ast, TokenReference<'ast>> {
+    ) -> TokenReference<'ast> {
         // Indent any comments leading a token, as these comments are technically part of the function body block
         let formatted_leading_trivia: Vec<Token<'ast>> = self.load_token_trivia(
             current_token.leading_trivia().collect(),
@@ -591,13 +639,43 @@ impl CodeFormatter {
             None,
         );
 
-        Cow::Owned(TokenReference::new(
+        // Special case for block end tokens:
+        // We will reverse the leading trivia, and keep removing any newlines we find, until we find something else, then we stop.
+        // This is to remove unnecessary newlines at the end of the block.
+        let mut iter = formatted_leading_trivia.iter().rev().peekable();
+
+        let mut formatted_leading_trivia = Vec::new();
+        let mut stop_removal = false;
+        while let Some(x) = iter.next() {
+            match x.token_type() {
+                TokenType::Whitespace { ref characters } => {
+                    if !stop_removal
+                        && characters.contains('\n')
+                        && !matches!(
+                            iter.peek().map(|x| x.token_kind()),
+                            Some(TokenKind::SingleLineComment) | Some(TokenKind::MultiLineComment)
+                        )
+                    {
+                        continue;
+                    } else {
+                        formatted_leading_trivia.push(x.to_owned());
+                    }
+                }
+                _ => {
+                    formatted_leading_trivia.push(x.to_owned());
+                    stop_removal = true; // Stop removing newlines once we have seen some sort of comment
+                }
+            }
+        }
+
+        // Need to reverse the vector since we reversed the iterator
+        formatted_leading_trivia.reverse();
+
+        TokenReference::new(
             formatted_leading_trivia,
-            Token::new(TokenType::Symbol {
-                symbol: Symbol::End,
-            }),
+            Token::new(current_token.token_type().to_owned()),
             formatted_trailing_trivia,
-        ))
+        )
     }
 }
 
@@ -644,9 +722,7 @@ impl<'ast> VisitorMut<'ast> for CodeFormatter {
         } else {
             // We have some comments in here, so we need to remove any trailing whitespace then add a single new line
             pop_until_no_whitespace(&mut formatted_leading_trivia);
-            formatted_leading_trivia.push(Token::new(TokenType::Whitespace {
-                characters: Cow::Owned(get_line_ending_character(&self.config.line_endings)),
-            }));
+            formatted_leading_trivia.push(self.create_newline_trivia());
 
             TokenReference::new(
                 formatted_leading_trivia,
