@@ -3,14 +3,12 @@ use crate::{
     IndentType,
 };
 #[cfg(feature = "luau")]
-use full_moon::ast::types::{AsAssertion, IndexedTypeInfo, TypeInfo, TypeSpecifier};
+use full_moon::ast::types::{IndexedTypeInfo, TypeAssertion, TypeInfo, TypeSpecifier};
 use full_moon::ast::{
-    span::ContainedSpan, BinOp, BinOpRhs, Call, Expression, FunctionArgs, FunctionBody,
-    FunctionCall, Index, MethodCall, Parameter, Prefix, Suffix, TableConstructor, UnOp, Value, Var,
-    VarExpression,
+    span::ContainedSpan, BinOp, Call, Expression, FunctionArgs, FunctionBody, FunctionCall, Index,
+    MethodCall, Parameter, Prefix, Suffix, TableConstructor, UnOp, Value, Var, VarExpression,
 };
-use full_moon::tokenizer::{Token, TokenKind, TokenReference, TokenType};
-use std::borrow::Cow;
+use full_moon::tokenizer::{Token, TokenReference, TokenType};
 
 /// Enum to determine how trivia should be added when using trivia formatter functions
 #[derive(Clone, Debug)]
@@ -23,43 +21,6 @@ pub enum FormatTriviaType<'ast> {
     NoChange,
 }
 
-macro_rules! move_binop_comments {
-    ($binop:expr, $trivia:expr, { $($operator:ident,)+ }) => {
-        match $binop.bin_op() {
-            $(
-                BinOp::$operator(token) => {
-                    let mut trailing_comments = token
-                        .trailing_trivia()
-                        .filter(|token| {
-                            token.token_kind() == TokenKind::SingleLineComment
-                                || token.token_kind()
-                                    == TokenKind::MultiLineComment
-                        })
-                        .map(|x| {
-                            // Prepend a single space beforehand
-                            vec![Token::new(TokenType::spaces(1)), x.to_owned()]
-                        })
-                        .flatten()
-                        .collect();
-
-                    // Move the comments over
-                    $trivia.append(&mut trailing_comments);
-
-                    // Recreate BinOp with no comments
-                    BinOp::$operator(Cow::Owned(token_reference_add_trivia(
-                        token.to_owned().into_owned(),
-                        FormatTriviaType::NoChange,
-                        FormatTriviaType::Replace(vec![Token::new(
-                            TokenType::spaces(1),
-                        )]),
-                    )))
-                }
-            )+
-            _ => panic!("unknown binop found"),
-        }
-    };
-}
-
 /// Returns a string presentation of a TokenReference with all trivia removed
 pub fn no_comments(token: &TokenReference) -> String {
     token.token().to_string()
@@ -68,7 +29,7 @@ pub fn no_comments(token: &TokenReference) -> String {
 impl CodeFormatter {
     /// Creates indent trivia without including `self.indent_level`.
     /// You should pass the exact amount of indent you require to this function
-    fn create_plain_indent_trivia<'ast>(&self, indent_level: usize) -> Token<'ast> {
+    pub fn create_plain_indent_trivia<'ast>(&self, indent_level: usize) -> Token<'ast> {
         match self.config.indent_type {
             IndentType::Tabs => Token::new(TokenType::tabs(indent_level)),
             IndentType::Spaces => {
@@ -80,7 +41,6 @@ impl CodeFormatter {
     fn expression_split_binop<'ast>(
         &self,
         expression: Expression<'ast>,
-        binop_leading_trivia: FormatTriviaType<'ast>,
         indent_increase: usize,
     ) -> Expression<'ast> {
         match expression {
@@ -103,154 +63,98 @@ impl CodeFormatter {
                     };
                 }
 
-                // Increase the indent level of the trivia
-                let mut current_indent_vec = match &binop_leading_trivia {
-                    FormatTriviaType::Append(vec) | FormatTriviaType::Replace(vec) => vec.to_vec(),
-                    FormatTriviaType::NoChange => {
-                        panic!("we are hanging an expression with no indent trivia")
-                    }
-                };
-
                 // Modify the parentheses to hang the expression
                 let (start_token, end_token) = contained.tokens();
-
+                // Create a newline after the start brace and before the end brace
+                // Also, indent enough for the first expression in the start brace
                 let contained = ContainedSpan::new(
-                    Cow::Owned(token_reference_add_trivia(
+                    token_reference_add_trivia(
                         start_token.to_owned(),
                         FormatTriviaType::NoChange,
-                        FormatTriviaType::Append({
-                            // Create a new line at the end of the start token, then indent enough for the first expression
-                            let mut new_vec = current_indent_vec.to_vec();
-                            new_vec.insert(0, self.create_newline_trivia());
-                            new_vec.push(self.create_plain_indent_trivia(1));
-                            new_vec
-                        }),
-                    )),
-                    Cow::Owned(token_reference_add_trivia(
+                        FormatTriviaType::Append(vec![
+                            self.create_newline_trivia(),
+                            self.create_plain_indent_trivia(indent_increase + 1),
+                        ]),
+                    ),
+                    token_reference_add_trivia(
                         end_token.to_owned(),
-                        FormatTriviaType::Append(current_indent_vec.to_vec()),
+                        FormatTriviaType::Append(vec![
+                            self.create_newline_trivia(),
+                            self.create_plain_indent_trivia(indent_increase),
+                        ]),
                         FormatTriviaType::NoChange,
-                    )),
+                    ),
                 );
 
-                // Modify the binop leading trivia to increment by one
-                current_indent_vec.push(self.create_plain_indent_trivia(1));
-                let binop_leading_trivia = match binop_leading_trivia {
-                    FormatTriviaType::Append(_) => FormatTriviaType::Append(current_indent_vec),
-                    FormatTriviaType::Replace(_) => FormatTriviaType::Replace(current_indent_vec),
-                    FormatTriviaType::NoChange => FormatTriviaType::NoChange,
-                };
-
                 Expression::Parentheses {
                     contained,
                     expression: Box::new(self.expression_split_binop(
                         *expression,
-                        binop_leading_trivia,
                         indent_increase + 1, // Apply indent increase
                     )),
                 }
             }
             Expression::UnaryOperator { unop, expression } => Expression::UnaryOperator {
                 unop,
-                expression: Box::new(self.expression_split_binop(
-                    *expression,
-                    binop_leading_trivia,
-                    indent_increase,
-                )),
+                expression: Box::new(self.expression_split_binop(*expression, indent_increase)),
             },
-            Expression::Value {
-                value,
-                binop,
-                #[cfg(feature = "luau")]
-                as_assertion,
-            } => {
-                // Need to check if there is a binop
-                let mut trailing_trivia = trivia_util::get_value_trailing_comments(&value);
-
-                let mut update_value = true;
-
+            Expression::BinaryOperator { lhs, binop, rhs } => {
                 let binop = match binop {
-                    Some(binop) => {
-                        // Don't add the trivia if the binop is binding
-                        let binop = match binop.bin_op() {
-                            BinOp::GreaterThan(_)
-                            | BinOp::GreaterThanEqual(_)
-                            | BinOp::LessThan(_)
-                            | BinOp::LessThanEqual(_)
-                            | BinOp::TildeEqual(_)
-                            | BinOp::TwoEqual(_) => {
-                                // Remove the new value because we don't want that anymore
-                                update_value = false;
-                                // Return original binop
-                                binop
-                            }
+                    // Don't add the trivia if the binop is binding
+                    BinOp::GreaterThan(_)
+                    | BinOp::GreaterThanEqual(_)
+                    | BinOp::LessThan(_)
+                    | BinOp::LessThanEqual(_)
+                    | BinOp::TildeEqual(_)
+                    | BinOp::TwoEqual(_) => {
+                        // Return original binop
+                        binop
+                    }
+                    _ => {
+                        // If there are any comments trailing the BinOp, we need to move them to before the BinOp
+                        let mut trailing_comments = trivia_util::binop_trailing_comments(&binop);
+                        // Create a newline just before the BinOp, and preserve the indentation
+                        trailing_comments.push(self.create_newline_trivia());
+                        trailing_comments.push(self.create_plain_indent_trivia(indent_increase));
 
-                            _ => {
-                                // Move any comments after the binop to trailing the value, otherwise we will create issues
-                                let new_bin_op = move_binop_comments!(binop, trailing_trivia, {
-                                    And,
-                                    Caret,
-                                    Minus,
-                                    Or,
-                                    Percent,
-                                    Plus,
-                                    Slash,
-                                    Star,
-                                    TwoDots,
-                                });
-
-                                let new_binop = binop_rhs_add_trivia(
-                                    BinOpRhs::new(new_bin_op, Box::new(binop.rhs().to_owned())),
-                                    binop_leading_trivia.to_owned(),
-                                    FormatTriviaType::NoChange,
-                                );
-                                new_binop
-                            }
-                        };
-
-                        let rhs = Box::new(self.expression_split_binop(
-                            binop.rhs().to_owned(),
-                            binop_leading_trivia.to_owned(),
-                            indent_increase,
-                        ));
-                        Some(binop.with_rhs(rhs))
+                        binop_add_trivia(
+                            binop,
+                            FormatTriviaType::Replace(trailing_comments),
+                            FormatTriviaType::Replace(vec![Token::new(TokenType::spaces(1))]),
+                        )
                     }
-                    None => None,
                 };
 
-                trailing_trivia.push(self.create_newline_trivia());
-                let new_value = match update_value {
-                    true => value_add_trailing_trivia(
-                        match *value {
-                            // Handle any values which may have expressions inside of them
-                            // which we may need to split onto multiple lines
-                            Value::ParseExpression(expression) => {
-                                Value::ParseExpression(self.expression_split_binop(
-                                    expression.to_owned(),
-                                    binop_leading_trivia.to_owned(),
-                                    indent_increase,
-                                ))
-                            }
-                            _ => *value,
-                        },
-                        FormatTriviaType::Replace(trailing_trivia),
-                    ),
-                    false => *value,
-                };
-
-                Expression::Value {
-                    value: Box::new(new_value),
+                Expression::BinaryOperator {
+                    lhs: Box::new(self.expression_split_binop(*lhs, indent_increase)),
                     binop,
-                    #[cfg(feature = "luau")]
-                    as_assertion,
+                    rhs: Box::new(self.expression_split_binop(*rhs, indent_increase)),
                 }
             }
+
+            Expression::Value {
+                value,
+                #[cfg(feature = "luau")]
+                type_assertion,
+            } => Expression::Value {
+                value: match *value {
+                    Value::ParenthesesExpression(expression) => {
+                        Box::new(Value::ParenthesesExpression(
+                            self.expression_split_binop(expression, indent_increase),
+                        ))
+                    }
+                    _ => value,
+                },
+                #[cfg(feature = "luau")]
+                type_assertion,
+            },
+
+            // Can't hang anything else, so just return the original expression
+            _ => expression,
         }
     }
 
-    // Splits an expression at its binops, pushing each binop part onto a newline
-    // Optionally, will also indent any further binops apart from the first one if an indent hang is wanted
-    pub fn hang_expression<'ast>(
+    pub fn hang_expression_no_trailing_newline<'ast>(
         &self,
         expression: Expression<'ast>,
         additional_indent_level: Option<usize>,
@@ -259,91 +163,63 @@ impl CodeFormatter {
         let additional_indent_level =
             additional_indent_level.unwrap_or(0) + hang_level.unwrap_or(0);
         let hang_level = self.indent_level + additional_indent_level;
-        let indent_trivia = self.create_plain_indent_trivia(hang_level);
 
-        self.expression_split_binop(
-            expression,
-            FormatTriviaType::Replace(vec![indent_trivia]),
-            additional_indent_level + 1,
-        )
+        self.expression_split_binop(expression, hang_level)
     }
 
-    /// Similar to `hang_expression`, except will not force a new line character at the very end of the expression
-    pub fn hang_expression_no_trailing_newline<'ast>(
+    pub fn hang_expression<'ast>(
         &self,
         expression: Expression<'ast>,
         additional_indent_level: Option<usize>,
         hang_level: Option<usize>,
     ) -> Expression<'ast> {
-        let expr = self.hang_expression(expression, additional_indent_level, hang_level);
-        let mut trailing_trivia = trivia_util::get_expression_trailing_trivia(&expr);
+        let expr = self.hang_expression_no_trailing_newline(
+            expression,
+            additional_indent_level,
+            hang_level,
+        );
 
-        // Remove last trivia, check if its whitespace, then add it back if not
-        if let Some(trivia) = trailing_trivia.pop() {
-            if trivia.token_kind() != TokenKind::Whitespace {
-                trailing_trivia.push(trivia)
-            }
-        }
-
-        expression_add_trailing_trivia(expr, FormatTriviaType::Replace(trailing_trivia))
+        expression_add_trailing_trivia(
+            expr,
+            FormatTriviaType::Append(vec![self.create_newline_trivia()]),
+        )
     }
 }
 
 // Remainder of Nodes
-macro_rules! binop_leading_trivia {
-    ($enum:ident, $value:ident, $leading_trivia:ident, { $($operator:ident,)+ }) => {
+macro_rules! binop_trivia {
+    ($enum:ident, $value:ident, $leading_trivia:ident, $trailing_trivia:ident, { $($operator:ident,)+ }) => {
         match $value {
             $(
-                $enum::$operator(token) => $enum::$operator(Cow::Owned(token_reference_add_trivia(token.into_owned(), $leading_trivia, FormatTriviaType::NoChange))),
+                $enum::$operator(token) => $enum::$operator(token_reference_add_trivia(token, $leading_trivia, $trailing_trivia)),
             )+
+            other => panic!("unknown node {:?}", other),
         }
     };
 }
 
-pub fn binop_rhs_add_trivia<'ast>(
-    binop_rhs: BinOpRhs<'ast>,
+fn binop_add_trivia<'ast>(
+    binop: BinOp<'ast>,
     leading_trivia: FormatTriviaType<'ast>,
     trailing_trivia: FormatTriviaType<'ast>,
-) -> BinOpRhs<'ast> {
-    let binop = if let FormatTriviaType::NoChange = leading_trivia {
-        binop_rhs.bin_op().to_owned()
-    } else {
-        let op = binop_rhs.bin_op().to_owned();
-        binop_leading_trivia!(BinOp, op, leading_trivia, {
-            And,
-            Caret,
-            GreaterThan,
-            GreaterThanEqual,
-            LessThan,
-            LessThanEqual,
-            Minus,
-            Or,
-            Percent,
-            Plus,
-            Slash,
-            Star,
-            TildeEqual,
-            TwoDots,
-            TwoEqual,
-        })
-    };
-
-    let rhs = std::boxed::Box::new(expression_add_trailing_trivia(
-        binop_rhs.rhs().to_owned(),
-        trailing_trivia,
-    ));
-    binop_rhs.with_bin_op(binop).with_rhs(rhs)
-}
-
-pub fn binop_rhs_add_trailing_trivia<'ast>(
-    binop_rhs: BinOpRhs<'ast>,
-    trailing_trivia: FormatTriviaType<'ast>,
-) -> BinOpRhs<'ast> {
-    let rhs = std::boxed::Box::new(expression_add_trailing_trivia(
-        binop_rhs.rhs().to_owned(),
-        trailing_trivia,
-    ));
-    binop_rhs.with_rhs(rhs)
+) -> BinOp<'ast> {
+    binop_trivia!(BinOp, binop, leading_trivia, trailing_trivia, {
+        And,
+        Caret,
+        GreaterThan,
+        GreaterThanEqual,
+        LessThan,
+        LessThanEqual,
+        Minus,
+        Or,
+        Percent,
+        Plus,
+        Slash,
+        Star,
+        TildeEqual,
+        TwoDots,
+        TwoEqual,
+    })
 }
 
 /// Adds trailing trivia at the end of a ContainedSpan node
@@ -354,16 +230,16 @@ pub fn contained_span_add_trivia<'ast>(
 ) -> ContainedSpan<'ast> {
     let (start_token, end_token) = contained_span.tokens();
     ContainedSpan::new(
-        Cow::Owned(token_reference_add_trivia(
+        token_reference_add_trivia(
             start_token.to_owned(),
             leading_trivia,
             FormatTriviaType::NoChange,
-        )),
-        Cow::Owned(token_reference_add_trivia(
+        ),
+        token_reference_add_trivia(
             end_token.to_owned(),
             FormatTriviaType::NoChange,
             trailing_trivia,
-        )),
+        ),
     )
 }
 
@@ -380,6 +256,7 @@ pub fn call_add_trailing_trivia<'ast>(
             method_call,
             trailing_trivia,
         )),
+        other => panic!("unknown node {:?}", other),
     }
 }
 
@@ -404,17 +281,21 @@ pub fn expression_add_leading_trivia<'ast>(
             unop: unop_add_leading_trivia(unop, leading_trivia),
             expression,
         },
+        Expression::BinaryOperator { lhs, binop, rhs } => Expression::BinaryOperator {
+            lhs: Box::new(expression_add_leading_trivia(*lhs, leading_trivia)),
+            binop,
+            rhs,
+        },
         Expression::Value {
             value,
-            binop,
             #[cfg(feature = "luau")]
-            as_assertion,
+            type_assertion,
         } => Expression::Value {
             value: Box::new(value_add_leading_trivia(*value, leading_trivia)),
-            binop,
             #[cfg(feature = "luau")]
-            as_assertion,
+            type_assertion,
         },
+        other => panic!("unknown node {:?}", other),
     }
 }
 
@@ -426,36 +307,24 @@ pub fn expression_add_trailing_trivia<'ast>(
     match expression {
         Expression::Value {
             value,
-            binop,
             #[cfg(feature = "luau")]
-            as_assertion,
+            type_assertion,
         } => {
             #[cfg(feature = "luau")]
-            if let Some(as_assertion) = as_assertion {
+            if let Some(as_assertion) = type_assertion {
                 return Expression::Value {
                     value,
-                    binop,
-                    as_assertion: Some(as_assertion_add_trailing_trivia(
+                    type_assertion: Some(type_assertion_add_trailing_trivia(
                         as_assertion,
                         trailing_trivia,
                     )),
                 };
             }
 
-            if let Some(binop) = binop {
-                Expression::Value {
-                    value,
-                    binop: Some(binop_rhs_add_trailing_trivia(binop, trailing_trivia)),
-                    #[cfg(feature = "luau")]
-                    as_assertion,
-                }
-            } else {
-                Expression::Value {
-                    value: Box::new(value_add_trailing_trivia(*value, trailing_trivia)),
-                    binop,
-                    #[cfg(feature = "luau")]
-                    as_assertion,
-                }
+            Expression::Value {
+                value: Box::new(value_add_trailing_trivia(*value, trailing_trivia)),
+                #[cfg(feature = "luau")]
+                type_assertion,
             }
         }
 
@@ -477,6 +346,13 @@ pub fn expression_add_trailing_trivia<'ast>(
             unop,
             expression: Box::new(expression_add_trailing_trivia(*expression, trailing_trivia)),
         },
+
+        Expression::BinaryOperator { lhs, binop, rhs } => Expression::BinaryOperator {
+            lhs,
+            binop,
+            rhs: Box::new(expression_add_trailing_trivia(*rhs, trailing_trivia)),
+        },
+        other => panic!("unknown node {:?}", other),
     }
 }
 
@@ -499,13 +375,11 @@ pub fn function_args_add_trailing_trivia<'ast>(
         },
 
         // Add for completeness
-        FunctionArgs::String(token_reference) => {
-            FunctionArgs::String(Cow::Owned(token_reference_add_trivia(
-                token_reference.into_owned(),
-                FormatTriviaType::NoChange,
-                trailing_trivia,
-            )))
-        }
+        FunctionArgs::String(token_reference) => FunctionArgs::String(token_reference_add_trivia(
+            token_reference,
+            FormatTriviaType::NoChange,
+            trailing_trivia,
+        )),
         FunctionArgs::TableConstructor(table_constructor) => {
             FunctionArgs::TableConstructor(table_constructor_add_trivia(
                 table_constructor,
@@ -513,6 +387,7 @@ pub fn function_args_add_trailing_trivia<'ast>(
                 trailing_trivia,
             ))
         }
+        other => panic!("unknown node {:?}", other),
     }
 }
 
@@ -522,11 +397,11 @@ pub fn function_body_add_trailing_trivia<'ast>(
     trailing_trivia: FormatTriviaType<'ast>,
 ) -> FunctionBody<'ast> {
     let function_body_token = function_body.end_token().to_owned();
-    function_body.with_end_token(Cow::Owned(token_reference_add_trivia(
+    function_body.with_end_token(token_reference_add_trivia(
         function_body_token,
         FormatTriviaType::NoChange,
         trailing_trivia,
-    )))
+    ))
 }
 
 /// Adds leading trivia to the start of a FunctionCall node
@@ -543,10 +418,8 @@ pub fn function_call_add_trailing_trivia<'ast>(
     function_call: FunctionCall<'ast>,
     trailing_trivia: FormatTriviaType<'ast>,
 ) -> FunctionCall<'ast> {
-    let mut new_suffixes: Vec<Suffix<'ast>> = function_call
-        .iter_suffixes()
-        .map(|x| x.to_owned())
-        .collect();
+    let mut new_suffixes: Vec<Suffix<'ast>> =
+        function_call.suffixes().map(|x| x.to_owned()).collect();
     if let Some(last_suffix) = new_suffixes.pop() {
         new_suffixes.push(suffix_add_trailing_trivia(
             last_suffix.to_owned(),
@@ -576,12 +449,9 @@ pub fn index_add_trailing_trivia<'ast>(
         },
         Index::Dot { dot, name } => Index::Dot {
             dot,
-            name: Cow::Owned(token_reference_add_trivia(
-                name.into_owned(),
-                FormatTriviaType::NoChange,
-                trailing_trivia,
-            )),
+            name: token_reference_add_trivia(name, FormatTriviaType::NoChange, trailing_trivia),
         },
+        other => panic!("unknown node {:?}", other),
     }
 }
 
@@ -604,16 +474,17 @@ pub fn parameter_add_trivia<'ast>(
     trailing_trivia: FormatTriviaType<'ast>,
 ) -> Parameter<'ast> {
     match parameter {
-        Parameter::Ellipse(token) => Parameter::Ellipse(Cow::Owned(token_reference_add_trivia(
-            token.into_owned(),
+        Parameter::Ellipse(token) => Parameter::Ellipse(token_reference_add_trivia(
+            token,
             leading_trivia,
             trailing_trivia,
-        ))),
-        Parameter::Name(token) => Parameter::Name(Cow::Owned(token_reference_add_trivia(
-            token.into_owned(),
+        )),
+        Parameter::Name(token) => Parameter::Name(token_reference_add_trivia(
+            token,
             leading_trivia,
             trailing_trivia,
-        ))),
+        )),
+        other => panic!("unknown node {:?}", other),
     }
 }
 
@@ -623,14 +494,15 @@ pub fn prefix_add_leading_trivia<'ast>(
     leading_trivia: FormatTriviaType<'ast>,
 ) -> Prefix<'ast> {
     match prefix {
-        Prefix::Name(token_reference) => Prefix::Name(Cow::Owned(token_reference_add_trivia(
-            token_reference.into_owned(),
+        Prefix::Name(token_reference) => Prefix::Name(token_reference_add_trivia(
+            token_reference,
             leading_trivia,
             FormatTriviaType::NoChange,
-        ))),
+        )),
         Prefix::Expression(expression) => {
             Prefix::Expression(expression_add_leading_trivia(expression, leading_trivia))
         }
+        other => panic!("unknown node {:?}", other),
     }
 }
 
@@ -642,6 +514,7 @@ pub fn suffix_add_trailing_trivia<'ast>(
     match suffix {
         Suffix::Call(call) => Suffix::Call(call_add_trailing_trivia(call, trailing_trivia)),
         Suffix::Index(index) => Suffix::Index(index_add_trailing_trivia(index, trailing_trivia)),
+        other => panic!("unknown node {:?}", other),
     }
 }
 
@@ -710,21 +583,22 @@ pub fn unop_add_leading_trivia<'ast>(
     leading_trivia: FormatTriviaType<'ast>,
 ) -> UnOp<'ast> {
     match unop {
-        UnOp::Hash(token_reference) => UnOp::Hash(Cow::Owned(token_reference_add_trivia(
-            token_reference.into_owned(),
+        UnOp::Hash(token_reference) => UnOp::Hash(token_reference_add_trivia(
+            token_reference,
             leading_trivia,
             FormatTriviaType::NoChange,
-        ))),
-        UnOp::Minus(token_reference) => UnOp::Minus(Cow::Owned(token_reference_add_trivia(
-            token_reference.into_owned(),
+        )),
+        UnOp::Minus(token_reference) => UnOp::Minus(token_reference_add_trivia(
+            token_reference,
             leading_trivia,
             FormatTriviaType::NoChange,
-        ))),
-        UnOp::Not(token_reference) => UnOp::Not(Cow::Owned(token_reference_add_trivia(
-            token_reference.into_owned(),
+        )),
+        UnOp::Not(token_reference) => UnOp::Not(token_reference_add_trivia(
+            token_reference,
             leading_trivia,
             FormatTriviaType::NoChange,
-        ))),
+        )),
+        other => panic!("unknown node {:?}", other),
     }
 }
 
@@ -734,34 +608,30 @@ pub fn value_add_leading_trivia<'ast>(
 ) -> Value<'ast> {
     match value {
         Value::Function((token, function_body)) => Value::Function((
-            Cow::Owned(token_reference_add_trivia(
-                token.into_owned(),
-                leading_trivia,
-                FormatTriviaType::NoChange,
-            )),
+            token_reference_add_trivia(token, leading_trivia, FormatTriviaType::NoChange),
             function_body,
         )),
         Value::FunctionCall(function_call) => Value::FunctionCall(
             function_call_add_leading_trivia(function_call, leading_trivia),
         ),
-        Value::Number(token_reference) => Value::Number(Cow::Owned(token_reference_add_trivia(
-            token_reference.into_owned(),
+        Value::Number(token_reference) => Value::Number(token_reference_add_trivia(
+            token_reference,
             leading_trivia,
             FormatTriviaType::NoChange,
-        ))),
-        Value::ParseExpression(expression) => {
-            Value::ParseExpression(expression_add_leading_trivia(expression, leading_trivia))
+        )),
+        Value::ParenthesesExpression(expression) => {
+            Value::ParenthesesExpression(expression_add_leading_trivia(expression, leading_trivia))
         }
-        Value::String(token_reference) => Value::String(Cow::Owned(token_reference_add_trivia(
-            token_reference.into_owned(),
+        Value::String(token_reference) => Value::String(token_reference_add_trivia(
+            token_reference,
             leading_trivia,
             FormatTriviaType::NoChange,
-        ))),
-        Value::Symbol(token_reference) => Value::Symbol(Cow::Owned(token_reference_add_trivia(
-            token_reference.into_owned(),
+        )),
+        Value::Symbol(token_reference) => Value::Symbol(token_reference_add_trivia(
+            token_reference,
             leading_trivia,
             FormatTriviaType::NoChange,
-        ))),
+        )),
         Value::TableConstructor(table_constructor) => {
             Value::TableConstructor(table_constructor_add_trivia(
                 table_constructor,
@@ -770,6 +640,7 @@ pub fn value_add_leading_trivia<'ast>(
             ))
         }
         Value::Var(var) => Value::Var(var_add_leading_trivia(var, leading_trivia)),
+        other => panic!("unknown node {:?}", other),
     }
 }
 
@@ -786,24 +657,24 @@ pub fn value_add_trailing_trivia<'ast>(
         Value::FunctionCall(function_call) => Value::FunctionCall(
             function_call_add_trailing_trivia(function_call, trailing_trivia),
         ),
-        Value::Number(token_reference) => Value::Number(Cow::Owned(token_reference_add_trivia(
-            token_reference.into_owned(),
+        Value::Number(token_reference) => Value::Number(token_reference_add_trivia(
+            token_reference,
             FormatTriviaType::NoChange,
             trailing_trivia,
-        ))),
-        Value::ParseExpression(expression) => {
-            Value::ParseExpression(expression_add_trailing_trivia(expression, trailing_trivia))
-        }
-        Value::String(token_reference) => Value::String(Cow::Owned(token_reference_add_trivia(
-            token_reference.into_owned(),
+        )),
+        Value::ParenthesesExpression(expression) => Value::ParenthesesExpression(
+            expression_add_trailing_trivia(expression, trailing_trivia),
+        ),
+        Value::String(token_reference) => Value::String(token_reference_add_trivia(
+            token_reference,
             FormatTriviaType::NoChange,
             trailing_trivia,
-        ))),
-        Value::Symbol(token_reference) => Value::Symbol(Cow::Owned(token_reference_add_trivia(
-            token_reference.into_owned(),
+        )),
+        Value::Symbol(token_reference) => Value::Symbol(token_reference_add_trivia(
+            token_reference,
             FormatTriviaType::NoChange,
             trailing_trivia,
-        ))),
+        )),
         Value::TableConstructor(table_constructor) => {
             Value::TableConstructor(table_constructor_add_trivia(
                 table_constructor,
@@ -812,6 +683,7 @@ pub fn value_add_trailing_trivia<'ast>(
             ))
         }
         Value::Var(var) => Value::Var(var_add_trailing_trivia(var, trailing_trivia)),
+        other => panic!("unknown node {:?}", other),
     }
 }
 
@@ -821,15 +693,16 @@ pub fn var_add_leading_trivia<'ast>(
     leading_trivia: FormatTriviaType<'ast>,
 ) -> Var<'ast> {
     match var {
-        Var::Name(token_reference) => Var::Name(Cow::Owned(token_reference_add_trivia(
-            token_reference.into_owned(),
+        Var::Name(token_reference) => Var::Name(token_reference_add_trivia(
+            token_reference,
             leading_trivia,
             FormatTriviaType::NoChange,
-        ))),
+        )),
         Var::Expression(var_expresion) => Var::Expression(var_expression_add_leading_trivia(
             var_expresion,
             leading_trivia,
         )),
+        other => panic!("unknown node {:?}", other),
     }
 }
 
@@ -839,15 +712,16 @@ pub fn var_add_trailing_trivia<'ast>(
     trailing_trivia: FormatTriviaType<'ast>,
 ) -> Var<'ast> {
     match var {
-        Var::Name(token_reference) => Var::Name(Cow::Owned(token_reference_add_trivia(
-            token_reference.into_owned(),
+        Var::Name(token_reference) => Var::Name(token_reference_add_trivia(
+            token_reference,
             FormatTriviaType::NoChange,
             trailing_trivia,
-        ))),
+        )),
         Var::Expression(var_expression) => Var::Expression(var_expression_add_trailing_trivia(
             var_expression,
             trailing_trivia,
         )),
+        other => panic!("unknown node {:?}", other),
     }
 }
 
@@ -866,10 +740,8 @@ pub fn var_expression_add_trailing_trivia<'ast>(
     trailing_trivia: FormatTriviaType<'ast>,
 ) -> VarExpression<'ast> {
     // TODO: This is copied from FunctionCall, can we combine them?
-    let mut new_suffixes: Vec<Suffix<'ast>> = var_expression
-        .iter_suffixes()
-        .map(|x| x.to_owned())
-        .collect();
+    let mut new_suffixes: Vec<Suffix<'ast>> =
+        var_expression.suffixes().map(|x| x.to_owned()).collect();
     if let Some(last_suffix) = new_suffixes.pop() {
         new_suffixes.push(suffix_add_trailing_trivia(
             last_suffix.to_owned(),
@@ -894,13 +766,11 @@ pub fn type_info_add_trailing_trivia<'ast>(
             );
             TypeInfo::Array { braces, type_info }
         }
-        TypeInfo::Basic(token_reference) => {
-            TypeInfo::Basic(Cow::Owned(token_reference_add_trivia(
-                token_reference.to_owned().into_owned(),
-                FormatTriviaType::NoChange,
-                trailing_trivia,
-            )))
-        }
+        TypeInfo::Basic(token_reference) => TypeInfo::Basic(token_reference_add_trivia(
+            token_reference.to_owned(),
+            FormatTriviaType::NoChange,
+            trailing_trivia,
+        )),
         TypeInfo::Callback {
             parentheses,
             arguments,
@@ -965,11 +835,11 @@ pub fn type_info_add_trailing_trivia<'ast>(
             base,
             question_mark,
         } => {
-            let question_mark = Cow::Owned(token_reference_add_trivia(
-                question_mark.to_owned().into_owned(),
+            let question_mark = token_reference_add_trivia(
+                question_mark.to_owned(),
                 FormatTriviaType::NoChange,
                 trailing_trivia,
-            ));
+            );
             TypeInfo::Optional {
                 base,
                 question_mark,
@@ -1006,6 +876,8 @@ pub fn type_info_add_trailing_trivia<'ast>(
             let right = Box::new(type_info_add_trailing_trivia(*right, trailing_trivia));
             TypeInfo::Union { left, pipe, right }
         }
+
+        other => panic!("unknown node {:?}", other),
     }
 }
 
@@ -1016,11 +888,11 @@ pub fn indexed_type_info_add_trailing_trivia<'ast>(
 ) -> IndexedTypeInfo<'ast> {
     match indexed_type_info {
         IndexedTypeInfo::Basic(token_reference) => {
-            IndexedTypeInfo::Basic(Cow::Owned(token_reference_add_trivia(
-                token_reference.to_owned().into_owned(),
+            IndexedTypeInfo::Basic(token_reference_add_trivia(
+                token_reference.to_owned(),
                 FormatTriviaType::NoChange,
                 trailing_trivia,
-            )))
+            ))
         }
         IndexedTypeInfo::Generic {
             base,
@@ -1036,16 +908,19 @@ pub fn indexed_type_info_add_trailing_trivia<'ast>(
                 generics,
             }
         }
+
+        other => panic!("unknown node {:?}", other),
     }
 }
 
 #[cfg(feature = "luau")]
-pub fn as_assertion_add_trailing_trivia<'ast>(
-    as_assertion: AsAssertion<'ast>,
+pub fn type_assertion_add_trailing_trivia<'ast>(
+    type_assertion: TypeAssertion<'ast>,
     trailing_trivia: FormatTriviaType<'ast>,
-) -> AsAssertion<'ast> {
-    let cast_to = type_info_add_trailing_trivia(as_assertion.cast_to().to_owned(), trailing_trivia);
-    as_assertion.with_cast_to(cast_to)
+) -> TypeAssertion<'ast> {
+    let cast_to =
+        type_info_add_trailing_trivia(type_assertion.cast_to().to_owned(), trailing_trivia);
+    type_assertion.with_cast_to(cast_to)
 }
 
 #[cfg(feature = "luau")]
