@@ -6,6 +6,8 @@ local y = ((1) * 3)
 local z = (...) == nil and foo or bar
 local foo = not (bar and baz)
 local bar = (not bar) and baz
+local cond = condition and (not object or object.Value == y)
+local baz = (-4 + 3) * 2
 
 ("hello"):format()
 
@@ -15,4 +17,8 @@ end
   
 print(x())
 print((x()))
-print(((x())))
\ No newline at end of file
+print(((x())))
+
+path = (function()
+  return true
+end)()
\ No newline at end of file
